package it.eng.configurazione.bean;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.Table;

import org.hibernate.annotations.CollectionOfElements;
import org.hibernate.annotations.LazyCollection;

@Entity
@Table(name="e_operation")

public class Operation {

	private int id;
	private String name=null;
	private DataSource dataSource=null;
	private Set<ExtractionProcess> process=new HashSet<ExtractionProcess>();
	private Set<Field> field=new HashSet<Field>();
	private Set<OperationParameter> parameter=new HashSet<OperationParameter>();

	@CollectionOfElements
	public Set<Field> getField() {
		return field;
	}
	public void setField(Set<Field> field) {
		this.field = field;
	}

	@CollectionOfElements
	public Set<OperationParameter> getParameter() {
		return parameter;
	}

	public void setParameter(Set<OperationParameter> parameters) {
		this.parameter = parameters;
	}
	public void addParameter(OperationParameter p){
		this.parameter.add(p);
		p.setOperation(this);
	}
	public void addField(Field p){
		this.field.add(p);
		p.setOperation(this);
	}

    @OneToOne(optional = false)
    @JoinColumn(name = "dataSource_id")
	public DataSource getDataSource() {
		return dataSource;
	}
	public void setDataSource(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	@ManyToMany(
			targetEntity=it.eng.configurazione.bean.ExtractionProcess.class,
			cascade={CascadeType.PERSIST, CascadeType.MERGE},
			fetch=FetchType.EAGER
			)
	@JoinTable(
			name="e_operation_process",
			joinColumns={@JoinColumn(name="operation_id")},
			inverseJoinColumns={@JoinColumn(name="process_id")}
			)	
	public Set<ExtractionProcess> getProcess() {
		return process;
	}
	public void setProcess(Set<ExtractionProcess> process) {
		this.process = process;
	}
	
	public void addProcess(ExtractionProcess event) {
		this.process.add(event);
	    event.getOperation().add(this);
	}

	public void removeProcess(ExtractionProcess event) {
		this.process.remove(event);
	    event.getOperation().remove(this);

	}
	
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	public int getId() {
		return id;
	}
	public void setId(int id) {
		this.id = id;
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 *  
	 * @param parName
	 * @return
	 */
	public String readParameterValue(String parName){
		Iterator<OperationParameter> iter=this.parameter.iterator();
		while (iter.hasNext()){
			OperationParameter tmp=iter.next();
			if (tmp.getName().equals(parName)) return tmp.getValue();
		}
		return null;
	}
	/**
	 *  
	 * @param parName
	 * @return
	 */
	public Field getField(String fieldName){
		Iterator<Field> iter=this.field.iterator();
		while (iter.hasNext()){
			Field tmp=iter.next();
			if (tmp!=null && tmp.getName().equals(fieldName)) return tmp;
		}
		return null;
	}
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + id;
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final Operation other = (Operation) obj;
		if (id != other.id)
			return false;
		return true;
	}	
}

